@@ -1,6 +1,6 @@
 __author__ = "Shyue Ping Ong, William Davidson Richards, Stephen Dacek, " \
              "Xiaohui Qu"
-__date__ = "Jun 13 2014"
-__version__ = "0.7.3"
+__date__ = "Jul 14 2014"
+__version__ = "0.7.4"
 
 from custodian import Custodian
